from os import environ
from typing import NamedTuple

from apexpy.exceptions import NotFoundError, PlayerNotFoundError, ServerError, UnauthorizedError


class Constants(NamedTuple):
    try:
        from dotenv import load_dotenv
        load_dotenv()

    except ModuleNotFoundError:
        pass

    try:
        API_KEY = environ['APEX_KEY']
    except KeyError:
        API_KEY = None

    BASE_URL = 'https://public-api.tracker.gg/apex/v1/standard/profile/'

    API_OK = 200

    API_ERROR_MAP = {

        401: UnauthorizedError,
        400: NotFoundError,
        403: UnauthorizedError,
        404: PlayerNotFoundError,
        500: ServerError,
    }

    PLATFORM_MAP = {
        'pc': 5,
        'xbox': 1,
        'psn': 2
    }



from apexpy.characters import ApexCharacter
from apexpy.httprequest import ApexRequest
from apexpy.utils import Constants


class ApexApi:
    """
    Represents the Apex api containing all the data requested by :class:`.ApexRequest`.

    Parameters
    ----------
        key : :class:`str`
            The apex legends api key.

    Attributes
    ----------
        name
            :class:`str`
            Player's name
        key
            :class:`str`
            api key
        legends
            List[:class:`.ApexCharacter`]
            Legends' data
        stats
            List[:class:`dict`]
            Player's general stats
        id
            :class:`str`
            Player's id

    """
    def __init__(self, key: str = None):

        self.name = self.platform = None
        self.key = key
        self.legends = []  # Game's characters are called 'legends' in this game.

    async def _populate(self, data) -> None:
        """
        Populates object with data

        :param data:
        :return: :class:`None`
        """

        self.legends = [ApexCharacter(char_data) for char_data in data['data']['children']]

        for k, v in data['data']['metadata'].items():
            setattr(self, k, v)

        stats = data['data']['stats']

        self.stats = [
            {stats['metadata']['key']: stats['value'], 'rank': stats.get('rank')} for stats in stats
        ]

    async def search(self, name: str, platform: str) -> None:
        """
        Creates the :class:`.ApexRequest` object that takes care of the api's communication.
        Automatically sets variables and call the population method.


        :param name: :class:`str`
        :param platform: :class:`str` Three platforms supported are pc, xbox and psn, they are transformed into their numeric code, 5, 2, 1 respectively.

        :return: :class:`None`
        """
        self.name = name
        self.platform = Constants.PLATFORM_MAP.get(platform)

        data = await ApexRequest(self.name, self.platform, api_key=self.key).session()
        await self._populate(await data.json())

    def __str__(self):
        return f'{self.__class__.__name__}({self.name})'




class CustomBaseException(Exception):
    """
    Base exception, exceptions are raised depending on the received code from the api.

    .. code-block:: python

        API_ERROR_MAP = { 401: UnauthorizedError, 400: NotFoundError, 403: UnauthorizedError, 404: PlayerNotFoundError, 500: ServerError}
    """
    def __init__(self, message=''):
        self.message = message

    def __str__(self):
        return self.message


class ApiKeyNotProvidedError(CustomBaseException):
    """
    Raised when the api key is not directly provided and doesn't exist in os.environ
    """
    pass


class UnauthorizedError(CustomBaseException):
    pass


class NotFoundError(CustomBaseException):
    pass


class PlayerNotFoundError(CustomBaseException):
    pass


class ServerError(CustomBaseException):
    pass




class ApexCharacter:
    """
    Represents an Apex legend character

    Parameters
    ----------
        data : :class:`dict`
            Every characters data.

    Attributes
    ----------
        raw_data
            :class:`dict`
            Raw data

        name
            :class:`str`
            Character's name

        id
            :class:`str`
            Every character id.

            Wraith -> legend_1,
            Bangalore -> legend_2,
            Caustic -> legend_3,
            Mirage -> legend_4,
            Bloodhound -> legend_5,
            Gibraltar -> legend_6,
            Lifeline -> legend_7,
            Pathfinder -> legend_8
        icon
            :class:`str`
            Character's icon image
        bgimage
            :class:`str`
            Character's background image
        stats
            List[:class:`dict`]
            Character's stats data
    """
    def __init__(self, data):
        metadata = data['metadata']
        stats = data['stats']

        self.raw_data = data

        self.id = data.get('id')

        self.name = metadata.get('legend_name')
        self.icon = metadata.get('icon')
        self.bgimage = metadata.get('bgimage')

        self.stats = [
            {stats['metadata']['key']: stats['value'], 'rank': stats.get('rank')} for stats in stats
        ]

    def __str__(self):
        return f'{self.__class__.__name__}({self.name})'



import aiohttp

from apexpy.exceptions import ApiKeyNotProvidedError
from apexpy.utils import Constants


class ApexRequest:
    """
    Represents the asynchronous http call made to the apex legends api

    Parameters
    ----------
        name : :class:`str`
            Player's name
        platform : :class:`str`
            Platform numeric code
        api_key : :class:`str`
            Apex key, if not provided directly it'll be looked in os.environ

    Attributes
    ----------

        platform
        name
        api_key
        headers
            :class:`dict`
            Headers sent with the call {'TRN-Api-KEY': key}
        self.req_url
            :class:`str`
            End point


    """
    def __init__(self, name: str, platform: int, api_key: str = None):

        if api_key is None and Constants.API_KEY is None:
            raise ApiKeyNotProvidedError

        self.platform = platform
        self.name = name

        self.api_key = api_key if api_key else Constants.API_KEY
        self.headers = {'TRN-Api-KEY': self.api_key}

        self.req_url = f'{Constants.BASE_URL}{self.platform}/{self.name}'
        self._raw_json = None

    @staticmethod
    async def _error_handler(resp: int) -> None:
        """
        If the api's response is other than 200 (OK) an error will be raised depending on the receive code.
        :param resp: :class:`int`
        :return: :class:`None`
        """
        if resp != Constants.API_OK:
            raise Constants.API_ERROR_MAP.get(resp)

    async def session(self) -> aiohttp.client.ClientResponse:
        """
        Makes the api call.
        :return: :class:`aiohttp.client.ClientResponse`
        """
        async with aiohttp.ClientSession(headers=self.headers) as session:
            async with session.get(self.req_url) as resp:
                await self._error_handler(resp.status)
                self._raw_json = await resp.json()
                return resp



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

with open('requirements.txt', 'r') as f:
    requirementse = f.read().splitlines()

setuptools.setup(
    name="apexlegendspy",
    version="0.0.1",
    author="surister",
    author_email="surister98@gmail.com",
    description="Apex-legends api async wrapper",
    license='MIT',
    long_description=long_description,
    include_package_data=True,
    python_requires='>=3.6.0',
    install_requires=[
        'aiohttp'
    ],
    long_description_content_type="text/markdown",
    url="https://github.com/surister/apexpy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Utilities"
    ],
)
